@@ -75,7 +75,7 @@ validateCdmSource <- function(src) {
 }
 
 validateX <- function(x, name, fun) {
-  if (!identical(attr(x, "tbl_name"), name)) {
+  if (!identical(tableName(x), name)) {
     cli::cli_abort("table name is not correctly assigned in {fun}")
   }
   if (!"cdm_table" %in% class(x)) {
@@ -87,6 +87,19 @@ validateX <- function(x, name, fun) {
 #' @export
 print.cdm_source <- function(x, ...) {
   cli::cli_inform(
-    "This is a {attr(x, 'source_type')} cdm source"
+    "This is a {sourceType(x)} cdm source"
   )
 }
+
+#' Get the source type of a cdm_source.
+#'
+#' @param cdmSource A cdm_source object.
+#'
+#' @return A character vector that defines the type of cdm_source.
+#'
+#' @export
+#'
+sourceType <- function(cdmSource) {
+  assertClass(cdmSource, "cdm_source")
+  attr(cdmSource, "source_type")
+}
